﻿// Program 2
// CIS 199-01
// Due: 3/30/2019
// M9460
// This application calculates the marginal tax rate and
// tax due for filers in 2019 tax year.
// Version 3
// Listens to radio buttons to apply income thresholds
// Calculates tax due through running total with each
// tier's portion of the tax.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Prog3
{
    public partial class Prog3Form : Form
    {
        // Tax Year 2019 Data
        // Taxable income thresholds for each status
        // Single Filers array
        private readonly int [] SINGLE = { 9700, 39475, 84200, 160725 , 204100, 510300, int.MaxValue };
        //Married Filing Separately array 
        private readonly int [] SEPERATELY = { 9700, 39475, 84200, 160725, 204100, 306175, int.MaxValue };
        //// Married Filing Jointly array 
        private readonly int[] JOINTLY = { 19400, 78950, 168400, 321450, 408200, 612350, int.MaxValue };
        // Head of Household array
        private readonly int[] HOH = { 13850, 52850, 84200, 160700, 204100, 510300, int.MaxValue };
       // Income threshold values array that apply to this filer
        private int [] threshold = { 0,0,0,0,0,0};

        public Prog3Form()
        {
            InitializeComponent();
        }

        // User has clicked the Calculate Tax button
        // Will calculate and display their marginal tax rate and tax due
        private void calcTaxBtn_Click(object sender, EventArgs e)
        {
            // Tax Year 2019 Data
            // The marginal tax rates using arrays 
            decimal[] Rates = { .10m, .12m, .22m , .24m,.32m ,.35m,.37m};
            

            int income; // Filer's taxable income (input)

            decimal marginalRate; // Filer's calculated marginal tax rate
            decimal tax;          // Filer's calculated income tax due
            decimal currentTax;   // Filer's tax for current tier

            if (int.TryParse(incomeTxt.Text, out income) && income >= 0)
            {
                // Calculate income tax due and find their marginal rate
                // Uses running total approach

                marginalRate = Rates[0]; // Assume lowest rate
                tax = Math.Min(income - 0, threshold[0] - 0) * Rates[0]; // Always some from Tier 1

                for (int i = 0; i < threshold.Length; i++)
                {
                    if (income > threshold[i]) // At least some from tier 2?
                    {
                        marginalRate = Rates[i+1];
                        currentTax = Math.Min((income - threshold[i]), (threshold[i+1]- threshold[i])) * Rates[i+1];
                        tax += currentTax;
                    }
                }

                // Output results
                marginalRateOutLbl.Text = $"{marginalRate:P0}";
                taxOutLbl.Text = $"{tax:C}";
            }
            else // Invalid input
                MessageBox.Show("Enter valid income!");
        }

        // Form is loading
        // Sets default filing status as Single
        private void Prog2Form_Load(object sender, EventArgs e)
        {
            singleRdoBtn.Checked = true; // Choose single by default
                                         // Will raise CheckedChanged event
        }

        // User has checked/unchecked Single radio button
        // Updates income thresholds
        private void singleRdoBtn_CheckedChanged(object sender, EventArgs e)
        {
            if (singleRdoBtn.Checked) // Single?
            {
                for (int i = 0; i < threshold.Length; i++)
                {
                    threshold[i] = SINGLE[i];
                }
            }
        }

        // User has checked/unchecked Married Filing Separately radio button
        // Updates income thresholds
        private void separatelyRdoBtn_CheckedChanged(object sender, EventArgs e)
        {
            if (separatelyRdoBtn.Checked) // Married Filing Separately?
            {
                for (int i = 0; i < threshold.Length; i++)
                {
                    threshold[i] = SEPERATELY[i];
                }
            }
        }

        // User has checked/unchecked Married Filing Jointly radio button
        // Updates income thresholds
        private void jointlyRdoBtn_CheckedChanged(object sender, EventArgs e)
        {
            if (jointlyRdoBtn.Checked) // Married Filing Jointly?
            {

                for (int i = 0; i < threshold.Length; i++)
                {
                    threshold[i] = JOINTLY[i];
                }
            }
        }

        // User has checked/unchecked Head of Household radio button
        // Updates income thresholds
        private void headOfHouseRdoBtn_CheckedChanged(object sender, EventArgs e)
        {
            if (headOfHouseRdoBtn.Checked) // Head of Household?
            {

                for (int i = 0; i < threshold.Length; i++)
                {
                    threshold[i] = HOH[i];
                }
            }
        }
    }
}
